@@ -56,6 +56,8 @@
 #define XtCRuleset	"Rules"
 #define XtNxpmdir	"xpmdir"
 #define XtCXpmdir	"Xpmdir"
+#define XtNxpmcls	"xpmcls"
+#define XtCXpmcls	"Xpmcls"
 #define XtNcardset	"cards"
 #define XtCCardset	"Cards"
 #define XtNfaceup	"faceup"
