@@ -11,6 +11,8 @@
 /*****************************************************************************/
 #include "X-pat.h"
 
+/* #define LABER */
+
 Display *dpy;
 int screen;
 unsigned long blackpixel;
@@ -18,6 +20,11 @@ unsigned long whitepixel;
 GC blackgc;
 GC whitegc;
 
+/* flush event queue */
+void flush_display(void) {
+    XSync(dpy, 0);	/* text first! */
+}
+
 #ifdef useXlib
 static int intersect(int x1, int w1, int x2, int w2) {
     if (x1 < x2)
@@ -72,22 +79,23 @@ void redraw_table(XExposeEvent *xev) {
 
 #ifdef LABER
     printf("request for expose %d,%d of size %d,%d, cnt %d\n", xev->x, xev->y, xev->width, xev->height, xev->count);
+    /* printf("height of 0=%d, 8=%d\n", p.geo[0], p.geo[8].h); */
 #endif
 
     for (i = 0; i < game.numpiles; ++i) {
-	struct pile *p;
+	struct pilegeometry *p;
 #ifdef SPEEDUP
 	if (pile_is_drawn[i]++)
 	    continue;
 #endif
-	p = graphic.pile + i;
+	p = geo.pg + i;
 	if (NOT_DISPLAYED(p) ||
-	    xev->y >= p->y + p->ytotal || xev->y + xev->height <= p->y ||
-	    xev->x >= p->x + p->xtotal || xev->x + xev->width <= p->x)
+	    xev->x >= p->x + p->w || xev->x + xev->width <= p->x ||
+	    xev->y >= p->y + p->h || xev->y + xev->height <= p->y)
 	    continue;	/* this pile is not affected */
 #ifdef LABER
 	printf("request for update pile %d of pos %d,%d, size %d,%d\n", i,
-	       p->x, p->y, p->xtotal, p->ytotal);
+	       p->x, p->y, p->w, p->h);
 #endif
 	draw_pileupdate(i, 0);
     }
@@ -112,16 +120,20 @@ void redraw_table(XExposeEvent *xev) {
 /* the pointer is somewhere in the area of pile i */
 /* check, if a card has to be exposed */
 
-static void expose_card(Pileindex i, int y) {
-    struct pile *p;
+static void expose_card(Pileindex i, int x, int y) {
+    struct pilegeometry *p;
     Cardindex ind;
 
     if (EMPTY(i))
 	return;
-    p = graphic.pile + i;
+    p = geo.pg + i;
+#if 0
     ind = game.ind[i];
     while (ind != INDEX_OF_LAST_CARD(i) && y >=	p->y + graphic.cardy[ind+1])
         ++ind;
+#else
+    ind = FindCardOfMousePos(i, x, y);
+#endif
     if (ind == INDEX_OF_LAST_CARD(i))   /* the most bottom card don't has to */
         ind = -1;                       /* be put on foreground */
     if (graphic.zoomed_card != ind) {   /* change of state */
@@ -138,13 +150,13 @@ void mouse_motion(XPointerMovedEvent *xev) {
     Pileindex i;
     /* find new pile */
     for (i = FIRST_SLOT; i <= LAST_SLOT; ++i) {
-	struct pile *p;
-	p = graphic.pile+i;
-	if (xev->x >= p->x && xev->x < p->x + p->xmaxwidth &&
-	    xev->y >= p->y && xev->y < p->y + p->ymaxheight) {
+	struct pilegeometry *p;
+	p = geo.pg + i;
+	if (xev->x >= p->x && xev->x < p->x + p->w &&
+	    xev->y >= p->y && xev->y < p->y + p->h) {
 	    /* yeah, a slot is hit */
 	    /* find out which card is the target */
-	    expose_card(i, xev->y);
+	    expose_card(i, xev->x - p->x, xev->y - p->y);
 	    return;
         }
     }
@@ -173,13 +185,14 @@ void button_press(XButtonPressedEvent *xev) {
 #endif
 
     for (i = 0; i < game.numpiles; ++i) {
-	struct pile *p;
-	p = graphic.pile+i;
+	struct pilegeometry *p;
+	p = geo.pg + i;
 	if (NOT_DISPLAYED(p))
 	    continue;
-	if (xev->x >= p->x && xev->x < p->x + p->xmaxwidth &&
-	    xev->y >= p->y && xev->y < p->y + p->ymaxheight) {
+	if (xev->x >= p->x && xev->x < p->x + p->w &&
+	    xev->y >= p->y && xev->y < p->y + p->h) {
 	    Cardindex cardi;
+#if 0
 	    if EMPTY(i)
 		cardi = -1;
 	    else {
@@ -190,6 +203,11 @@ void button_press(XButtonPressedEvent *xev) {
 			   graphic.cardy[cardi] > xev->y - graphic.pile[i].y)
 			--cardi;
 	    }
+#else
+	    cardi = FindCardOfMousePos(i, xev->x - p->x, xev->y - p->y);
+	    if (cardi == -2)	/* below := lowest */
+		cardi = INDEX_OF_LAST_CARD(i);
+#endif
 	    switch (xev->button) {
 	    case Button1:            /* quick move */
 		button_pressed(i, cardi, 1);
@@ -203,14 +221,11 @@ void button_press(XButtonPressedEvent *xev) {
 #else
 #ifndef useXview
 		if (game.piletype[i] == Slot)
-		    expose_card(i, xev->y);
+		    expose_card(i, xev->x - p->x, xev->y - p->y);
 #else
-		if (game.piletype[i] == Slot &&
-		    xev->y < graphic.pile[i].y + graphic.cardy[cardi] +
-			CARD_HEIGHT)
-		    expose_card(i, xev->y);
-		else
-		{
+		if (game.piletype[i] == Slot && cardi >= 0) {
+		    expose_card(i, xev->x - p->x, xev->y - p->y);
+		} else {
 		    menu_show(cmenu, canvas, bp_event, 0);
 		    return;
 		}
@@ -242,15 +257,17 @@ void button_press(XButtonPressedEvent *xev) {
 
 void cmd_MinWindow(void) {
     XSize_t w, h;
-    w = graphic.min_width;
-    h = graphic.min_height;
+    w = geo.min_width;
+    h = geo.min_height;
+/*    printf("Force resize to min (%d,%d)\n", w, h); */
     Force_Resize(w, h);
 }
 
 void cmd_PreferredWindow(void) {
     XSize_t w, h;
-    w = graphic.preferred_width;
-    h = graphic.preferred_height;
+    w = geo.preferred_width;
+    h = geo.preferred_height;
+/*    printf("Force resize to prf (%d,%d)\n", w, h); */
     Force_Resize(w, h);
 }
 
@@ -322,7 +339,7 @@ void resize_event(XSize_t w, XSize_t h) {
 	Pileindex i;
 	/* fix piles    THIS IS WRONG FOR 2-ROW LAYOUTS!!!! FIX THIS! */
 	for (i = 0; i < rules.numslots; ++i)
-	    graphic.pile[SLOT(i)].ymaxheight = graphic.height - graphic.pile[SLOT(i)].y;
+	    geo.pg[XPATSLOT(i)].h = geo.h - geo.pg[XPATSLOT(i)].y;
 	
 	for (i = FIRST_SLOT; i <= LAST_SLOT; ++i)
 	    if (pile_resize(i))
