@@ -205,7 +205,7 @@ static void make_GCs(unsigned long red, unsigned long cbcolor) {
     
     cardbackgc = XCreateGC(dpy, RootWindow(dpy, screen), gcflags, &gcv);
     
-    if (graphic.is_color)	{
+    if (graphic.is_color) {
 	gcv.foreground = cbcolor;
 	gcv.background = WhitePixel(dpy, screen);
 	gcflags = GCForeground | GCBackground;
@@ -213,7 +213,7 @@ static void make_GCs(unsigned long red, unsigned long cbcolor) {
 	XCopyPlane(dpy, tmpmap, logomap, logogc, 0, 0, 
 		   logo_width, logo_height, 0, 0, 1);
 	XFreeGC(dpy, logogc);
-    } else	{
+    } else {
 	XCopyPlane(dpy, tmpmap, logomap, whitegc, 0, 0, 
 		   logo_width, logo_height, 0, 0, 1);
     }
@@ -347,82 +347,8 @@ static Pixmap make_color_map(char *bits, int width, int height, Suit suit)
     return (newmap);
 }
 
-
 #ifdef LOADCARDS
-static Pixmap card_bitmaps[68];	/* these hold pixmaps of full depth */
-                                /* we have room for 8 jokers */
-static int numcards;		/* when reading external cardsets */
-static int inset;
-
-/* read a complete set of card bitmaps from a file */
-static void read_bitmap(const char *file)
-{   FILE *fp;
-    int i, w, h, bpl, cardsize;
-    char *card_bits;
-    char fullname[200];
-    struct {			 /* Note: all hi...-fields are currently 0 */
-	char type;               /* 0 for cards without and 1 for cards with an frame 	*/
-	char hitype;
-	char num;                /* # of cards (without jokers) in file 		*/
-	char hinum;
-	char x;                  /* width of one card 					*/
-	char hix;
-	char y;                  /* height of one card on screen (size in file can differ) */
-	char hiy;
-	char rx;                 /* horizontal half axe of rounded corner 		*/
-	char hirx;
-	char ry;                 /* vertical half axe of rounded corner 		*/
-	char hiry;
-	char stddelta;           /* fraction of height for displaying 			*/
-	char histddelta;
-	char numjokers;          /* # of different joker cards 			*/
-	char hinumjokers;
-	char numcardbacks;       /* # of different cardbacks 				*/
-	char hinumcardbacks;
-	char fill[14];           /* reserved for future requirements 			*/
-    } hdr;
-
-    if (!strchr(file, '/') &&
-	strlen(file) + strlen(LIBDIR) + 8 <= sizeof(fullname)) {
-	sprintf(fullname, "%s/%s.cards", LIBDIR, file);
-	file = fullname;
-    }
-    fp = fopen(file, "r");
-    if (!fp || fread(&hdr, sizeof(hdr), 1, fp) != 1) {
-	fprintf(stderr, "cannot read file %s\n", file);
-	exit(EXIT_FAILURE);
-    }
-    /* check the number of cards in the header */
-    if (hdr.num < 52 || hdr.num > 57) {
-	fprintf(stderr, "Bad value of numcards in file %s\n", file);
-	exit(EXIT_FAILURE);
-    }
-    w = hdr.x; h = hdr.y; numcards = hdr.num;
-    CARD_WIDTH = w;
-    CARD_HEIGHT = h;
-    ROUND_W = hdr.rx;           /* default radii of rounded corners */
-    ROUND_H = hdr.ry;
-    STD_DELTA = hdr.stddelta;
-    bpl = (w + 7) >> 3;         /* bytes per line */
-    if (!(card_bits = malloc(bpl * h * numcards))) {
-	fprintf(stderr, "Not enough memory for external cardset (file %s)\n",
-		file);
-	exit(EXIT_FAILURE);
-    }
-
-    /* read the card bitmaps */
-    fread(card_bits, bpl * h * numcards, 1, fp);
-    fclose(fp);
-
-    /* hdr.type & 1 is the number of pixels omitted in the bitmap */
-    cardsize = (h - 2 * (inset = (hdr.type & 1))) * bpl;
-
-    /* turn data into bitmaps */
-    for (i = 0; i < numcards; i++)	{
-	card_bitmaps[i] = XCreateBitmapFromData(dpy, 
-		RootWindow(dpy, screen), card_bits+i*cardsize, w-2*inset, h-2*inset);
-    }
-}
+#include "X-loadcards.c"
 #endif
 
 /* build jack to king bitmaps of the internal cardset by mirroring the first half */
@@ -491,7 +417,8 @@ static void build_internal_cardset(void)
 }
 
 void init_cards(const char *cardset, int rx, int ry,
-    unsigned long red, unsigned long cbcolor, int mem, const char *xpm) {
+		unsigned long red, unsigned long cbcolor, int mem,
+		const char *xpm, int closeness) {
 #ifdef NO_ROUND_CARDS
     rx = ry = 0;
 #endif
@@ -544,11 +471,6 @@ void init_cards(const char *cardset, int rx, int ry,
     if (ry >= 0)
 	card.ry = ry;
     
-    /* adapt arrows to match cardset sizes */
-    graphic.aw = graphic.ah = STD_DELTA / 6;
-    graphic.ya_h = graphic.xa_w = graphic.aw * 2;
-    graphic.xa_h = graphic.ya_w = graphic.aw * 4;
-
     card.back_delta_x = (CARD_WIDTH - logo_width)/2;
     card.back_delta_y = (CARD_HEIGHT - logo_height)/2;
 
@@ -563,18 +485,8 @@ void init_cards(const char *cardset, int rx, int ry,
 	     long gcflags, f, b;
 	     GC clipgc;
 	     
-#if 0
-	     if (DefaultDepth(dpy, screen) >= 8) {	/* I don't know why! */
-		 b = WhitePixel(dpy, screen);
-		 f = BlackPixel(dpy, screen);
-	     } else {
-		 f = WhitePixel(dpy, screen);
-		 b = BlackPixel(dpy, screen);
-	     }
-#else
 	     b = 0;
 	     f = 1;
-#endif
 	     /* first, clear the clipmap */
 	     /* (is it possible to do it easier?) */
 	     gcv.foreground = b;
@@ -628,10 +540,7 @@ void init_cards(const char *cardset, int rx, int ry,
 
 
 
-static void paint_cardback(int x, int y, int delta, Drawable d)
-{
-        if (!delta)
-            delta = CARD_HEIGHT;
+static void paint_cardback(int x, int y, int w, int h, Drawable d) {
 
 #define	INSETa	1
 #define INSETb  1
@@ -648,43 +557,43 @@ static void paint_cardback(int x, int y, int delta, Drawable d)
 #ifndef NO_ROUND_CARDS
 	if (ROUND_W) {
 
-	    XmuFillRoundedRectangle(dpy, d, blackgc, x, y, CARD_WIDTH-1, 
-		(delta == CARD_HEIGHT) ? delta-1 : delta + ROUND_H * 2,
+	    XmuFillRoundedRectangle(dpy, d, blackgc, x, y,
+		(w == CARD_WIDTH)  ? w-1 : w + ROUND_W * 2,
+		(h == CARD_HEIGHT) ? h-1 : h + ROUND_H * 2,
 		ROUND_W, ROUND_H);
 	} else
 #endif
-	    XFillRectangle(dpy, d, blackgc, x, y, CARD_WIDTH,
-		(delta == CARD_HEIGHT) ? delta : delta);
+	    XFillRectangle(dpy, d, blackgc, x, y, w, h);
 
 	XSetTSOrigin(dpy, backgc, x + card.back_delta_x, y + card.back_delta_y);
 
 #ifndef NO_ROUND_CARDS
 	if (ROUND_W)
 	    XmuFillRoundedRectangle(dpy, d, backgc, x + INSETa, y + INSETa, 
-		CARD_WIDTH - INSETa - INSETb - 1,
-		(delta == CARD_HEIGHT) ? 
-			(CARD_HEIGHT - INSETa - INSETb - 1) : delta + ROUND_H * 2,
+		(w == CARD_WIDTH  ? CARD_WIDTH  - INSETa - INSETb - 1 : w + ROUND_W * 2),
+		(h == CARD_HEIGHT ? CARD_HEIGHT - INSETa - INSETb - 1 : h + ROUND_H * 2),
 		ROUND_W, ROUND_H);
 	else
 #endif
 	    XFillRectangle(dpy, d, backgc, x + INSETa, y + INSETa, 
-		CARD_WIDTH - INSETa - INSETb - 1,
-		(delta == CARD_HEIGHT) ? 
-			(CARD_HEIGHT - INSETa - INSETb - 1) : delta-1);
+		(w == CARD_WIDTH  ? CARD_WIDTH  - INSETa - INSETb - 1 : w - 1),
+		(h == CARD_HEIGHT ? CARD_HEIGHT - INSETa - INSETb - 1 : h - 1));
 	/* correction */
 #ifndef NO_ROUND_CARDS
 	if (ROUND_W) {
-	    XmuDrawRoundedRectangle(dpy, d, blackgc, x, y, CARD_WIDTH-1, 
-		(delta == CARD_HEIGHT) ? delta-1 : delta + ROUND_H * 2,
-		ROUND_W, ROUND_H);
-	    XmuDrawRoundedRectangle(dpy, d, blackgc, x+1, y+1, CARD_WIDTH-3, 
-		(delta == CARD_HEIGHT) ? delta-3 : delta + ROUND_H * 2,
-		ROUND_W-1, ROUND_H-1);
+	    XmuDrawRoundedRectangle(dpy, d, blackgc, x, y,
+				    (w == CARD_WIDTH  ? w-1 : w + ROUND_W * 2),
+				    (h == CARD_HEIGHT ? h-1 : h + ROUND_H * 2),
+				    ROUND_W, ROUND_H);
+	    XmuDrawRoundedRectangle(dpy, d, blackgc, x+1, y+1,
+				    (w == CARD_WIDTH  ? w-3 : w + ROUND_W * 2),
+				    (h == CARD_HEIGHT ? h-3 : h + ROUND_H * 2),
+				    ROUND_W-1, ROUND_H-1);
 	} else
 #endif
-	    XDrawRectangle(dpy, d, blackgc, x+1, y+1, CARD_WIDTH-2,
-		(delta == CARD_HEIGHT) ? delta-2 : delta);
-
+	    XDrawRectangle(dpy, d, blackgc, x+1, y+1,
+			   (w == CARD_WIDTH  ? w-2 : w),
+			   (h == CARD_HEIGHT ? h-2 : h));
 }
 
 
@@ -891,47 +800,14 @@ static void paint_joker(int x, int y, Drawable d)
 	/* draw border on card */
 	XDrawRectangle(dpy, d, blackgc, x, y, CARD_WIDTH-1, CARD_HEIGHT-1);
 }
-/*
- * delta is 0 if the card is fully showing
- */
-static void paint_large_card(int x, int y, int rank, Suit suit, int delta, Drawable d)
-{
+
+static void paint_large_card(int x, int y, int rank, Suit suit, Drawable d) {
 	if (suit == Spade || suit == Club)	{
 		cardgc = blackgc;
 	} else	{
 		cardgc = redgc;
 	}
 
-	if (delta) {
-		cliprects[0].height = delta + ROUND_H;
-		XSetClipRectangles(dpy, cardgc, x, y, cliprects, 1, Unsorted);
-#ifndef NO_ROUND_CARDS
-		if (ROUND_W)	{
-		    XSetClipRectangles(dpy, whitegc, x, y, cliprects, 1, 
-						Unsorted);
-		    if (cardgc != blackgc)
-			XSetClipRectangles(dpy, blackgc, x, y, cliprects, 1,
-						Unsorted);
-
-		    /* add in ROUND_H to height so only the top is rounded */
-		    /* fill the background */
-		    XmuFillRoundedRectangle(dpy, d, whitegc, x, y, 
-			CARD_WIDTH-1, delta + ROUND_H, ROUND_W, ROUND_H);
-		    /* draw border on card */
-		    XmuDrawRoundedRectangle(dpy, d, blackgc, x, y, 
-			CARD_WIDTH-1, delta + ROUND_H, ROUND_W, ROUND_H);
-		} else	
-#endif
-		{
-		    /* fill the background */
-		    XFillRectangle(dpy, d, whitegc, x, y, 
-					CARD_WIDTH-1, delta);
-		    /* draw border on card */
-		    XDrawRectangle(dpy, d, blackgc, x, y, 
-					CARD_WIDTH-1, delta);
-		}
-		card_is_clipped = True;
-	} else	{	/* fill all the card */
 #ifndef NO_ROUND_CARDS
 		if (ROUND_W)	{
 		    /* fill the background */
@@ -953,7 +829,6 @@ static void paint_large_card(int x, int y, int rank, Suit suit, int delta, Drawa
 			CARD_WIDTH-1, CARD_HEIGHT-1);
 		}
 		card_is_clipped = False;
-	}
 
 	switch (rank)	{
 	case	King:
@@ -1016,7 +891,7 @@ static void paint_large_card(int x, int y, int rank, Suit suit, int delta, Drawa
 
 #ifndef NO_ROUND_CARDS
 	if (ROUND_W) {
-	    if (!delta && !x &&!y) {
+	    if (!x &&!y) {
 		/* special: if round card and initial call, repaint rounding */
 		/* this is for extremely rounded card which get over-painted */
 		XmuDrawRoundedRectangle(dpy, d, blackgc, x, y, 
@@ -1029,56 +904,8 @@ static void paint_large_card(int x, int y, int rank, Suit suit, int delta, Drawa
 #endif
 }
 
-#ifdef LOADCARDS
-static void paint_external_card(int x, int y, int cardnumber, int delta) {
-    if (cardnumber == CARDBACK)
-	cardgc = backgc;
-    else if (SUIT(cardnumber) == Spade || SUIT(cardnumber) == Club) {
-	cardgc = graphic.is_color ? blackgc: whitegc;
-    } else {
-	cardgc = redgc;
-    }
-    if (delta && !ROUND_W) {
-	cliprects[0].height = delta;
-	XSetClipRectangles(dpy, cardgc, x, y, cliprects, 1, Unsorted);
-	if (inset)
-	    XSetClipRectangles(dpy, blackgc, x, y, cliprects, 1, Unsorted);
-    }
 
-#ifndef NO_ROUND_CARDS
-    if (ROUND_W) {
-	XSetClipMask(dpy, cardgc, cardclipmap);
-	XSetClipOrigin(dpy, cardgc, x, y);
-	if (inset) {
-	    XSetClipMask(dpy, blackgc, cardclipmap);
-	    XSetClipOrigin(dpy, blackgc, x, y);
-	}
-    }
-#endif
-
-    XCopyPlane(dpy, card_bitmaps[cardnumber], table, cardgc, 0, 0, 
-	CARD_WIDTH-2*inset, CARD_HEIGHT-2*inset, x+inset, y+inset, 1);
-
-    if (inset) {	/* must draw border */
-#ifndef NO_ROUND_CARDS
-        if (ROUND_W)
-  	    XmuDrawRoundedRectangle(dpy, table, blackgc, x, y, 
-		CARD_WIDTH-1, CARD_HEIGHT-1, ROUND_W, ROUND_H);
-	else
-#endif
-	    XDrawRectangle(dpy, table, blackgc, x, y, CARD_WIDTH-1, CARD_HEIGHT-1);
-    }
-
-    if (delta || ROUND_W) {
-	XSetClipMask(dpy, cardgc, None);
-	if (inset)
-	    XSetClipMask(dpy, blackgc, None);
-    }
-}
-#endif
-
-
-void PaintCard(int x, int y, int c, int delta) {
+void PaintCard(int x, int y, int c, int w, int h) {
     static Suit Suittab[4] = { Club, Spade, Heart, Diamond };
     static int initial_call = 1;   /* initial-call-flag */
 
@@ -1093,10 +920,14 @@ void PaintCard(int x, int y, int c, int delta) {
 	    if (IS_JOKER(c))
 		c = SUIT(c) + SUITSYMBOL;	/* do this instead */
 	    if (c < numcards) {
-		paint_external_card(x, y, c, delta);
+		paint_external_card(x, y, c);
 		return;
 	    } else if (c == CARDBACK) {
-		paint_cardback(x, y, delta, table);
+		    if (!w)
+			w = CARD_WIDTH;
+		    if (!h)
+			h = CARD_HEIGHT;
+		paint_cardback(x, y, w, h, table);
 		return;
 	    } else {	/* must be suit symbol */
 		c = OUTLINE;	/* do outline as replacement */
@@ -1109,11 +940,13 @@ void PaintCard(int x, int y, int c, int delta) {
 		if (IS_JOKER(c))
 		    paint_joker(x, y, table);	/* do this instead */
 		else if (c != CARDBACK)
-		    paint_large_card(x, y, RANK(c), Suittab[SUIT(c)], delta, table);
+		    paint_large_card(x, y, RANK(c), Suittab[SUIT(c)], table);
 		else {
-		    if (!delta)
-			delta = CARD_HEIGHT;
-		    paint_cardback(x, y, delta, table);
+		    if (!w)
+			w = CARD_WIDTH;
+		    if (!h)
+			h = CARD_HEIGHT;
+		    paint_cardback(x, y, w, h, table);
 		}
 		return;
 	    } else {
@@ -1137,9 +970,9 @@ void PaintCard(int x, int y, int c, int delta) {
 			cardpicts[i] = XCreatePixmap(dpy, table,
 		 	     CARD_WIDTH, CARD_HEIGHT, DefaultDepth(dpy, screen));
 			if (i == CARDBACK)
-			    paint_cardback(0, 0, CARD_HEIGHT, cardpicts[i]);
+			    paint_cardback(0, 0, CARD_WIDTH, CARD_HEIGHT, cardpicts[i]);
 			else
-			    paint_large_card(0, 0, RANK(i), Suittab[SUIT(i)], 0, cardpicts[i]);
+			    paint_large_card(0, 0, RANK(i), Suittab[SUIT(i)], cardpicts[i]);
 		    }
 #ifdef SAVE_IMAGES
 		    for (i = 0; i < 52; ++i) {
@@ -1166,11 +999,14 @@ void PaintCard(int x, int y, int c, int delta) {
 			}
 		    }
 		}
-		delta += ROUND_H;
-		if (delta == ROUND_H || delta > CARD_HEIGHT)
-		    delta = CARD_HEIGHT;
+		w += ROUND_W;
+		h += ROUND_H;
+		if (w == ROUND_W || w > CARD_WIDTH)
+		    w = CARD_WIDTH;
+		if (h == ROUND_H || h > CARD_HEIGHT)
+		    h = CARD_HEIGHT;
 		XSetClipOrigin(dpy, cardbackgc, x, y);
-		XCopyArea(dpy, cardpicts[c], table, cardbackgc, 0, 0, CARD_WIDTH, delta, x, y);
+		XCopyArea(dpy, cardpicts[c], table, cardbackgc, 0, 0, w, h, x, y);
 		return;
 	    }
 	}
