@@ -17,12 +17,26 @@
 
 #include "X-pat.h"
 #include <time.h>
-#include <xpm.h>
+/* #include <xpm.h> */
+/* It seems that xpm.h is in the X11 subdirectory for most systems */
+#include <X11/xpm.h>
 
 #ifndef NO_ROUND_CARDS
 #include <X11/Xmu/Drawing.h>
+
+static Pixmap cardclipmap;
 #endif
 
+#ifdef LOADCARDS
+#include "gray1.bm"
+static GC redgc;
+static GC backgc;
+static GC cardgc;		/* gc in use when drawing cards */
+
+#include "X-loadcards.c"
+#endif
+
+
 /* with this module, cards are built the following way:
    1) If a complete card image does exist (filename: "(rank).(suit).xpm"),
       this image is read and used as the card pixmap. A rounded rectangle
@@ -102,12 +116,13 @@ struct cardconfig {
 static void read_cardconfig(const char *filename) {
     FILE *fp;
     int tmp[9];
-    if (!(fp = fopen(filename, "r")))
+    if (!(fp = fopen(filename, "r"))) {
+	fprintf(stderr, "xpat2: WARNING, cannot load configuration file %s\n", filename);
 	goto stdconfig;
-
+    }
     if (fscanf(fp, "%d %d", &CARD_WIDTH, &CARD_HEIGHT) != 2 ||
-	CARD_WIDTH < 49 || CARD_WIDTH > 123 ||
-	CARD_HEIGHT < 95 || CARD_HEIGHT > 200) {
+	CARD_WIDTH < 48 || CARD_WIDTH > 123 ||
+	CARD_HEIGHT < 64 || CARD_HEIGHT > 200) {
 	fclose(fp);
 	fp = NULL;
 	goto stdconfig;
@@ -192,6 +207,7 @@ static void read_cardconfig(const char *filename) {
 #define MAXJOKERS 4
 
 static Pixmap card_map[60+MAXJOKERS], card_clip;
+static int use_closeness = 0;
 
 /* we use a 2-phase algorithm, a combination of two mirrors */
 /* the amount of data copied is greater, but there are far less calls to XCopyArea */
@@ -221,11 +237,12 @@ static GC suitgc, rankgc;	/* gc in use when drawing suit and rank symbols */
 
 #include "logo.bm"
 
-static void make_GCs(void) {
+static void make_GCs(unsigned long red, unsigned long cbcolor) {
     XGCValues gcv;
     long gcflags;
     Pixmap tmpmap;
     Pixmap logomap;
+    GC logogc;
 
     /* make GC for cardbacks */
     tmpmap = XCreateBitmapFromData(dpy, RootWindow(dpy, screen),
@@ -241,6 +258,59 @@ static void make_GCs(void) {
     cardbackgc = XCreateGC(dpy, RootWindow(dpy, screen), gcflags, &gcv);
     suitgc = XCreateGC(dpy, RootWindow(dpy, screen), gcflags, &gcv);
     rankgc = XCreateGC(dpy, RootWindow(dpy, screen), gcflags, &gcv);
+#ifdef LOADCARDS
+    gcv.graphics_exposures = False;
+    /* make GC for RED */
+    if (graphic.is_color) {
+	gcv.foreground = red;
+	gcv.background = WhitePixel(dpy, screen);
+	gcflags = GCForeground | GCBackground | GCGraphicsExposures;
+	redgc = XCreateGC(dpy, RootWindow(dpy, screen), gcflags, &gcv);
+    } else {
+	gcv.tile = XCreateBitmapFromData(dpy, RootWindow(dpy, screen),
+		 gray1_bits, gray1_width, gray1_height);
+	gcv.fill_style = FillTiled;
+	gcv.foreground = BlackPixel(dpy, screen);
+	gcv.background = WhitePixel(dpy, screen);
+	
+	gcflags = GCTile | GCForeground | GCBackground |
+	    GCFillStyle | GCGraphicsExposures;
+	redgc = XCreateGC(dpy, RootWindow(dpy, screen), gcflags, &gcv);
+    }
+
+    /* make GC for cardbacks */
+    tmpmap = XCreateBitmapFromData(dpy, RootWindow(dpy, screen),
+	logo_bits, logo_width, logo_height);
+    logomap = XCreatePixmap(dpy, RootWindow(dpy, screen),
+	logo_width, logo_height, DefaultDepth(dpy, screen));
+    
+    gcv.foreground = BlackPixel(dpy, screen);
+    gcv.background = WhitePixel(dpy, screen);
+    gcv.graphics_exposures = True;
+    gcflags = GCForeground | GCBackground | GCGraphicsExposures;
+    
+    /*    cardbackgc = XCreateGC(dpy, RootWindow(dpy, screen), gcflags, &gcv); */
+    
+    if (graphic.is_color) {
+	gcv.foreground = cbcolor;
+	gcv.background = WhitePixel(dpy, screen);
+	gcflags = GCForeground | GCBackground;
+	logogc = XCreateGC(dpy, RootWindow(dpy, screen), gcflags, &gcv);
+	XCopyPlane(dpy, tmpmap, logomap, logogc, 0, 0, 
+		   logo_width, logo_height, 0, 0, 1);
+	XFreeGC(dpy, logogc);
+    } else {
+	XCopyPlane(dpy, tmpmap, logomap, whitegc, 0, 0, 
+		   logo_width, logo_height, 0, 0, 1);
+    }
+    XFreePixmap(dpy, tmpmap);
+    
+    gcv.tile = logomap;
+    gcv.fill_style = FillTiled;
+    gcflags |= GCTile | GCFillStyle | GCGraphicsExposures;
+    
+    backgc = XCreateGC(dpy, RootWindow(dpy, screen), gcflags, &gcv);
+#endif
 }
 
 
@@ -250,16 +320,81 @@ static char *xpmname(const char *s, const char *ext) {
     return buff;
 }
 
+#ifdef LOADCARDS
+#define	INSETa	1
+#define INSETb  1
+
+static void paint_cardback(int x, int y, int w, int h, Drawable d) {
+
+    /* change the origin so cards will have the same back anywhere
+     * on the table
+     */
+    /*
+     * there should be a tile centered in the card, with the
+     * surrounding tiles being partial
+     */
+    XSetClipMask(dpy, backgc, None);
+    XSetClipMask(dpy, blackgc, None);
+
+#ifndef NO_ROUND_CARDS
+    if (ROUND_W) {
+	XmuFillRoundedRectangle(dpy, d, blackgc, x, y,
+				(w == CARD_WIDTH)  ? w-1 : w + ROUND_W * 2,
+				(h == CARD_HEIGHT) ? h-1 : h + ROUND_H * 2,
+				ROUND_W, ROUND_H);
+    } else
+#endif
+	XFillRectangle(dpy, d, blackgc, x, y, w, h);
+
+    XSetTSOrigin(dpy, backgc, x + card.back_delta_x, y + card.back_delta_y);
+
+#ifndef NO_ROUND_CARDS
+    if (ROUND_W)
+	XmuFillRoundedRectangle(dpy, d, backgc, x + INSETa, y + INSETa,
+				(w == CARD_WIDTH  ? CARD_WIDTH  - INSETa - INSETb - 1 : w + ROUND_W * 2),
+				(h == CARD_HEIGHT ? CARD_HEIGHT - INSETa - INSETb - 1 : h + ROUND_H * 2),
+				ROUND_W, ROUND_H);
+    else
+#endif
+	XFillRectangle(dpy, d, backgc, x + INSETa, y + INSETa, 
+		       (w == CARD_WIDTH  ? CARD_WIDTH  - INSETa - INSETb - 1 : w - 1),
+		       (h == CARD_HEIGHT ? CARD_HEIGHT - INSETa - INSETb - 1 : h - 1));
+    /* correction */
+#ifndef NO_ROUND_CARDS
+    if (ROUND_W) {
+	XmuDrawRoundedRectangle(dpy, d, blackgc, x, y,
+				(w == CARD_WIDTH  ? w-1 : w + ROUND_W * 2),
+				(h == CARD_HEIGHT ? h-1 : h + ROUND_H * 2),
+				ROUND_W, ROUND_H);
+	XmuDrawRoundedRectangle(dpy, d, blackgc, x+1, y+1,
+				(w == CARD_WIDTH  ? w-3 : w + ROUND_W * 2),
+				(h == CARD_HEIGHT ? h-3 : h + ROUND_H * 2),
+				ROUND_W-1, ROUND_H-1);
+    } else
+#endif
+	XDrawRectangle(dpy, d, blackgc, x+1, y+1,
+		       (w == CARD_WIDTH  ? w-2 : w),
+		       (h == CARD_HEIGHT ? h-2 : h));
+}
+#endif
 
 void init_cards(const char *cardset, int rx, int ry,
-    unsigned long red, unsigned long cbcolor, int mem, const char *xpm) {
+		unsigned long red, unsigned long cbcolor, int mem,
+		const char *xpm, int closeness) {
     int i;
-
+    use_closeness = closeness;
     for (i = 0; i < 58; ++i)
 	card_map[i] = 0;
 
     mem_option = mem;
-    make_GCs();
+    card.cardset = cardset;
+    make_GCs(red, cbcolor);
+#ifdef LOADCARDS
+    if (cardset) {	/* read external cardset */
+	read_bitmap(cardset);
+	return;
+    }
+#endif
 #ifdef NO_ROUND_CARDS
     rx = ry = 0;
 #endif
@@ -268,14 +403,27 @@ void init_cards(const char *cardset, int rx, int ry,
 	s = malloc(strlen(LIBDIR) + 10);
 	sprintf(s, "%s/default", LIBDIR);
 	xpmdir = s;
+    } else if (*xpm == '~' && xpm[1] == '/') {	/* relative to HOME */
+	char *s;
+	const char *s2;
+	s2 = getenv("HOME");
+	if (!s2)
+	    s2 = "";	/* use path relative to root dir */
+	s = malloc(strlen(s2) + strlen(xpm) + 1);
+	sprintf(s, "%s%s", s2, xpm);
+	xpmdir = s;
     } else if (*xpm != '.' && *xpm != '/') {	/* supply path */
 	char *s;
 	s = malloc(strlen(LIBDIR) + strlen(xpm) + 2);
 	sprintf(s, "%s/%s", LIBDIR, xpm);
 	xpmdir = s;
+    } else {		/* has full or relative path */
+	char *s;
+	s = malloc(strlen(xpm) + 1);
+	strcpy(s, xpm);
+	xpmdir = s;
     }
     
-    card.cardset = cardset;
     /* build internal cardset */
     read_cardconfig(xpmname("Cards", "conf"));
 
@@ -295,11 +443,6 @@ void init_cards(const char *cardset, int rx, int ry,
     ROUND_W = ROUND_H = 9;
     STD_DELTA = cc.ydelta + SUITSIZE1;
 
-    /* adapt arrows to match cardset sizes */
-    graphic.aw = graphic.ah = STD_DELTA / 6;
-    graphic.ya_h = graphic.xa_w = graphic.aw * 2;
-    graphic.xa_h = graphic.ya_w = graphic.aw * 4;
-
     /* finally, compute a clipping mask for the cards */
     /* apply this clip mask to cardbackgc */
 #ifndef NO_ROUND_CARDS
@@ -368,7 +511,7 @@ void init_cards(const char *cardset, int rx, int ry,
 #ifdef SAVE_IMAGES
 static void write_xpmfile(char *filename, Pixmap pict, Pixmap clip) {
     if (XpmWriteFileFromPixmap(dpy, filename, pict, clip, NULL) != XpmSuccess)
-        fprintf(stderr, "error writing xpm file %s\n", filename);
+        fprintf(stderr, "xpat2: error writing xpm file %s\n", filename);
 }
 #endif
 
@@ -383,18 +526,23 @@ static struct picture {
 static int read_xpmfile(const char *name, struct picture *p) {
     XpmAttributes attribs;
     attribs.valuemask = 0;
+
+    if (use_closeness) {	/* allow allocation of 'near' colours */
+	attribs.valuemask |= XpmCloseness;
+	attribs.closeness = use_closeness;
+    }
     switch (XpmReadFileToPixmap(dpy, table, xpmname(name, "xpm"), &p->map, &p->clip, &attribs)) {
     case XpmSuccess:
 	break;	/* yeah! */
     case XpmColorError:
     case XpmColorFailed:
-	fprintf(stderr, "xpm: Not enough colors.\n");
+	fprintf(stderr, "xpat2: xpm: Not enough colours. Try allowing near colours (option -xpmcls).\n");
 	return p->flag = 0;
     case XpmFileInvalid:
-	fprintf(stderr, "xpm: Invalid file.\n");
+	fprintf(stderr, "xpat2: xpm: Invalid file.\n");
 	return p->flag = 0;
     case XpmNoMemory:
-	fprintf(stderr, "xpm: Out of memory.\n");
+	fprintf(stderr, "xpat2: xpm: Out of memory.\n");
 	return p->flag = 0;
     case XpmOpenFailed:
     default:
@@ -419,6 +567,10 @@ static int overlap_picture(const char *name, int two, Pixmap d) {
     if (!read_xpmfile(name, &p))
 	return 0;
 
+    if (two == 2 && 2 * p.h > CARD_HEIGHT) {
+	two = 0;	/* just a single picture */
+	MirrorCard(d, CARD_WIDTH, CARD_HEIGHT);
+    }
     if (two) {
 	x = (CARD_WIDTH-p.w) / 2;
 	y = CARD_HEIGHT/2 - p.h;
@@ -437,6 +589,8 @@ static int overlap_picture(const char *name, int two, Pixmap d) {
 	XCopyArea(dpy, p.map, d, whitegc, 0, 0, p.w, p.h, x, y);
     
     free_picture(&p);
+    if (two == 2)
+	MirrorCard(d, CARD_WIDTH, CARD_HEIGHT);
     return 1;
 }
 
@@ -608,7 +762,8 @@ static void paint_large_2(Rank rank, Suit suit, Drawable d) {
 	} else {
 	    char name[32];
 	    sprintf(name, "%s.%s.pic", US_rank_name[rank], US_suit_name[suit]);
-	    overlap_picture(name, 2, d);
+	    if (overlap_picture(name, 2, d))
+		return;	/* has done the mirroring */
 	}
     }
     MirrorCard(d, CARD_WIDTH, CARD_HEIGHT);
@@ -626,11 +781,11 @@ static void read_images(void) {
     }
     read_xpmfile("Background", &bg);
     if (!read_xpmfile("Suits", &p_suit)) {
-	fprintf(stderr, "cannot read suits\n");
+	fprintf(stderr, "xpat2: cannot read suits\n");
 	exit(1);
     }
     if (!read_xpmfile("Ranks", &p_rank)) {
-	fprintf(stderr, "cannot read ranks\n");
+	fprintf(stderr, "xpat2: cannot read ranks\n");
 	exit(1);
     }
     XSetClipMask(dpy, suitgc, p_suit.clip);
@@ -716,7 +871,6 @@ static void finish_card(int c, Pixmap d) {
 	    XDrawRectangle(dpy, d, blackgc, 0, 0, CARD_WIDTH-1, CARD_HEIGHT-1);
 }
 
-
 static void get_card(int c) {
     /* try to load card image from xpm file */
     char name[200];
@@ -728,10 +882,12 @@ static void get_card(int c) {
 	} else if (c >= JOKER) {
 	    sprintf(name, "Joker%d", 1 + ((c - JOKER) + (int)((game.seed / cc.numbacks) & 0xfff))
 		    % cc.numjokers);	/* use all different jokers, starting at rnd position */
+	} else if (c < 56) {
+	    sprintf(name, "Empty.%s", US_suit_name[SUIT(c)]);
 	} else
 	    *name = '\0';
     } else
-	sprintf(name, "%s.%s", US_rank_name[RANK(c)],
+	sprintf(name, "%s.%s.pic", US_rank_name[RANK(c)],
 		US_suit_name[SUIT(c)]);
 
     card_map[c] = start_card(c);
@@ -740,8 +896,23 @@ static void get_card(int c) {
 	finish_card(c, card_map[c]);
 	return;
     }
+    if (c == CARDBACK) {
+	/* second try with fallback cardback */
+	fprintf(stderr, "xpat2: cannot read %s, trying fallback cardback\n", name);
+	if (!overlap_picture("Cardback0", 0, card_map[c])) {
+	    /* if nothing works, try a very rudimetary one! (white area) */
+	    /* XClearArea(dpy, card_map[c], 0, 0, CARD_WIDTH, CARD_HEIGHT, False); */
+	    int i;
+	    XFillRectangle(dpy, card_map[c], whitegc, 0, 0, CARD_WIDTH, CARD_HEIGHT);
+	    for (i = 5; i < CARD_WIDTH/2-2; i+=4)
+		XDrawRectangle(dpy, card_map[c], blackgc, i, i,
+			       CARD_WIDTH-2*i-1, CARD_HEIGHT-2*i-1);
+	}
+	finish_card(c, card_map[c]);
+	return;
+    }
     if (c >= 56) {
-	fprintf(stderr, "cannot read %s\n", name);
+	fprintf(stderr, "xpat2: cannot read %s\n", name);
 	exit(1);
     }
 
@@ -758,11 +929,11 @@ static void get_card(int c) {
 #endif
 }
 
-void PaintCard(int x, int y, int c, int delta) {
+void PaintCard(int x, int y, int c, int w, int h) {
     
 #ifdef ALLATONCE
     static int loaded = 0;
-    if (!loaded) {
+    if (!loaded && !card.cardset) {
 	int i;
 	loaded = 1;
 	/* show_message(TXT_LOADING);
@@ -770,6 +941,7 @@ void PaintCard(int x, int y, int c, int delta) {
 	/* read all the cards we need in any case. Do not read */
 	/* Cardbacks or Jokers, they are possibly not needed */
 	/* printf("before read_images: %ld\n", time(NULL)); */
+
 	read_images();
 	for (i = 0; i < 56; ++i) {
 	    /* if (!(i&3)) printf("before card %d: %ld\n", i, time(NULL)); */
@@ -788,6 +960,28 @@ void PaintCard(int x, int y, int c, int delta) {
     if (c == NOTHING)
 	/* return; */
 	goto outline;	/* this is better */
+
+#ifdef LOADCARDS
+    if (card.cardset) {		/* externally loaded cards from bit images */
+	if (IS_JOKER(c))
+	    c = SUIT(c) + SUITSYMBOL; /* do this instead */
+	if (c < numcards) {
+	    paint_external_card(x, y, c);
+	    return;
+	} else if (c == CARDBACK) {
+	    if (!w)
+		w = CARD_WIDTH;
+	    if (!h)
+		h = CARD_HEIGHT;
+	    paint_cardback(x, y, w, h, table);
+	    return;
+	} else {		/* must be suit symbol */
+	    c = OUTLINE;	/* do outline as replacement */
+	    /* and fall thru */
+	}
+    }
+#endif
+
     /* JOKERs not yet fully implemented */
     if (IS_JOKER(c))
 	c = JOKER + (c-JOKER) % MAXJOKERS;
@@ -795,11 +989,14 @@ void PaintCard(int x, int y, int c, int delta) {
 	if (!card_map[c])
 	    /* must make cardmap */
 	    get_card(c);
-	delta += ROUND_H;
-	if (delta == ROUND_H || delta > CARD_HEIGHT)
-	    delta = CARD_HEIGHT;
+	w += ROUND_W;
+	h += ROUND_H;
+	if (w == ROUND_W || w > CARD_WIDTH)
+	    w = CARD_WIDTH;
+	if (h == ROUND_H || h > CARD_HEIGHT)
+	    h = CARD_HEIGHT;
 	XSetClipOrigin(dpy, cardbackgc, x, y);
-	XCopyArea(dpy, card_map[c], table, cardbackgc, 0, 0, CARD_WIDTH, delta, x, y);
+	XCopyArea(dpy, card_map[c], table, cardbackgc, 0, 0, w, h, x, y);
 	return;
     }
  outline:
