@@ -8,7 +8,8 @@
 /*									     */
 /*									     */
 /*****************************************************************************/
-#include "X-pat.h"
+
+#include "xpat.h"
 
 #ifndef AUDIO_DEVICE
 #define AUDIO_DEVICE "/dev/audio"
@@ -24,7 +25,7 @@ void play_sound(const char *filename) {
 	    audio = 0;
 	    return;		/* cannot open /dev/audio */
 	}
-	XSync(dpy, 0);	/* text first! */
+	flush_display();
 	sprintf(fullname, "%s/audio/%s.au", LIBDIR, filename);
 	if (!(fp = fopen(fullname, "rb"))) {
 	    fclose(fsnd);
