@@ -1,4 +1,7 @@
-/* -*- C++ -*-	Time-stamp: <06 May 99 12:11:06 Michael Bischoff> */
+/* -*- C++ -*-	Time-stamp: <05 Nov 00 21:10:54 Michael Bischoff> */
+/* Changes:
+   changed CopyROP to Qt::CopyROP for Qt 2.x
+   */
 #include "XQtCards.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -112,8 +115,8 @@ static void read_cardconfig(const char *filename) {
 	goto stdconfig;
     }
     if (fscanf(fp, "%d %d", &CARD_WIDTH, &CARD_HEIGHT) != 2 ||
-	CARD_WIDTH < 49 || CARD_WIDTH > 123 ||
-	CARD_HEIGHT < 95 || CARD_HEIGHT > 200) {
+	CARD_WIDTH < 48 || CARD_WIDTH > 123 ||
+	CARD_HEIGHT < 64 || CARD_HEIGHT > 200) {
 	fclose(fp);
 	fp = NULL;
 	goto stdconfig;
@@ -291,7 +294,7 @@ static int overlap_picture(const char *name, int two, QPixmap *d, QPainter *pain
 	x = (CARD_WIDTH-w) / 2;
 	y = (CARD_HEIGHT-h) / 2;
     }
-    bitBlt(d, x, y, p, 0, 0, w, h, CopyROP, FALSE);
+    bitBlt(d, x, y, p, 0, 0, w, h, Qt::CopyROP, FALSE);
     delete p;
 
     if (two == 2)
@@ -387,7 +390,7 @@ void XQtCards::local_constructor(const char *xpmdir) {
 	if (bw && bh && (((cc.flags & PIC_BG) || !(rank >= 10 && rank <= 12)))) {
 	    /* move background */
 	    bitBlt(pm_cards[i], bxo, byo, &Background,
-		0, 0, bw, bh, CopyROP, FALSE);
+		0, 0, bw, bh, Qt::CopyROP, FALSE);
 	}
 	paint.drawRoundRect(0, 0, w, h, rxp, ryp);
 	if (i < 52) {
@@ -396,26 +399,26 @@ void XQtCards::local_constructor(const char *xpmdir) {
 	        y = (rank % 5) * RANKHEIGHT;
         	// if (p_rank.clip) XSetClipOrigin(dpy, rankgc, cc.Rx-x, cc.Ry-y);
 	        bitBlt(pm_cards[i], cc.Rx, cc.Ry, &Ranks, x, y,
-			RANKWIDTH, RANKHEIGHT, CopyROP, FALSE);
+			RANKWIDTH, RANKHEIGHT, Qt::CopyROP, FALSE);
 	        if (cc.flags & TWORANKS)
 		    bitBlt(pm_cards[i], CARD_WIDTH-RANKWIDTH-cc.Rx, cc.Ry, &Ranks, x, y,
-                        RANKWIDTH, RANKHEIGHT, CopyROP, FALSE);
+                        RANKWIDTH, RANKHEIGHT, Qt::CopyROP, FALSE);
 	    }
 	    if (cc.Sx) {
         	/* Draw the upper small suit symbol(s). */
 	        x = cc.Sx;
 	        y = cc.Sy;
         	bitBlt(pm_cards[i], x, y, &Suits, suit * SUITSIZE3, SUITOFF3,
-		       SUITSIZE3, SUITSIZE3,  CopyROP, FALSE);
+		       SUITSIZE3, SUITSIZE3,  Qt::CopyROP, FALSE);
 	        if (cc.flags & TWORANKS)
 		    bitBlt(pm_cards[i], CARD_WIDTH-SUITSIZE3 - x, y, &Suits, suit * SUITSIZE3, SUITOFF3,
-			   SUITSIZE3, SUITSIZE3,  CopyROP, FALSE);
+			   SUITSIZE3, SUITSIZE3,  Qt::CopyROP, FALSE);
   	    }
 	    if (rank >= 10) {
 		/* picture */
 		if (cc.Mx > 0) {
 		    bitBlt(pm_cards[i], SUITSIZE2 * suit, SUITOFF2, &Suits, SUITSIZE2, SUITSIZE2,
-			   cc.Mx, cc.My, CopyROP, FALSE);
+			   cc.Mx, cc.My, Qt::CopyROP, FALSE);
 		}
 		/* horizontal line in the midst of the card: */
 		if (!(cc.flags & NOCENTERLINE))
@@ -434,7 +437,7 @@ void XQtCards::local_constructor(const char *xpmdir) {
 			y = CARD_HEIGHT/2 - bigh;
 			if (cc.flags & NOCENTERLINE)
 			    ++y;
-			bitBlt(pm_cards[i], x, y, &Pictures, bigw*suit, bigh*(rank-10), bigw, bigh, CopyROP, FALSE);
+			bitBlt(pm_cards[i], x, y, &Pictures, bigw*suit, bigh*(rank-10), bigw, bigh, Qt::CopyROP, FALSE);
 		    } else {
 			sprintf(name, "%s.%s.pic", US_rank_name[rank], US_suit_name[suit]);
 			if (overlap_picture(xpmfilename(directory, name),
@@ -460,10 +463,10 @@ void XQtCards::local_constructor(const char *xpmdir) {
 		    dl = CARD_WIDTH-RANKWIDTH;
 		    cx = dl-cc.Rx;
 		    cy = CARD_HEIGHT-cc.Ry-RANKHEIGHT;
-		    bitBlt(pm_cards[i], cx, cy, &Ranks, x, y, RANKWIDTH, RANKHEIGHT, CopyROP, FALSE);
+		    bitBlt(pm_cards[i], cx, cy, &Ranks, x, y, RANKWIDTH, RANKHEIGHT, Qt::CopyROP, FALSE);
 		    if (cc.flags & TWORANKS) {
 			cx = dl - cx;
-			bitBlt(pm_cards[i], cx, cy, &Ranks, x, y, RANKWIDTH, RANKHEIGHT, CopyROP, FALSE);
+			bitBlt(pm_cards[i], cx, cy, &Ranks, x, y, RANKWIDTH, RANKHEIGHT, Qt::CopyROP, FALSE);
 		    }
 		}
 		if (cc.Sx) {
@@ -474,11 +477,11 @@ void XQtCards::local_constructor(const char *xpmdir) {
 		    cx = dl-cc.Sx;
 		    cy = CARD_HEIGHT-cc.Sy-SUITSIZE3;
 		    bitBlt(pm_cards[i], cx, cy, &Suits, x, y,
-			   SUITSIZE3, SUITSIZE3,  CopyROP, FALSE);
+			   SUITSIZE3, SUITSIZE3,  Qt::CopyROP, FALSE);
 		    if (cc.flags & TWORANKS) {
 			cx = dl - cx;
 			bitBlt(pm_cards[i], cx, cy, &Suits, x, y,
-			       SUITSIZE3, SUITSIZE3,  CopyROP, FALSE);
+			       SUITSIZE3, SUITSIZE3,  Qt::CopyROP, FALSE);
 		    }
 		}
 #endif
@@ -491,7 +494,7 @@ void XQtCards::local_constructor(const char *xpmdir) {
 		/* first, all symbols which are not mirrored */
 
 #define PAINT_AT(xx, yy) {                                      \
-	       bitBlt(pm_cards[i], xx, yy, &Suits, x, SUITSIZE0, SUITSIZE1, SUITSIZE1, CopyROP, FALSE); \
+	       bitBlt(pm_cards[i], xx, yy, &Suits, x, SUITSIZE0, SUITSIZE1, SUITSIZE1, Qt::CopyROP, FALSE); \
 								    }
 
 
@@ -535,10 +538,15 @@ void XQtCards::local_constructor(const char *xpmdir) {
 	    } /* ace to ten */
 	} else {
 	    if (i < 56) {	// is a suit symbol
+		/* load picture instead of standard symbols? */
+		sprintf(name, "Empty.%s", US_suit_name[suit]);
+		if (overlap_picture(xpmfilename(directory, name),
+				    0, pm_cards[i], &paint))
+		    goto picture_done;
 		x = (CARD_WIDTH-SUITSIZE0)/2;
 		y = (CARD_HEIGHT-SUITSIZE0)/2;
 		bitBlt(pm_cards[i], x, y, &Suits, SUITSIZE0 * suit, 0,
-		       SUITSIZE0, SUITSIZE0, CopyROP, FALSE);
+		       SUITSIZE0, SUITSIZE0, Qt::CopyROP, FALSE);
 
 	    }
 	}
