@@ -23,6 +23,7 @@
 #include <X11/Xaw/Form.h>
 #include <X11/Xaw/Sme.h>
 #include <X11/Xaw/SmeBSB.h>
+#include <unistd.h>
 
 void create_help(void);
 void popup_help(void);
@@ -31,18 +32,27 @@ void popdown_help(Widget, XtPointer, XtPointer);
 static int help_active = 0;
 static Widget help, helppaned, helppanel, helptext, helpclose;
 
+#define TXT_HELPFILES		(xpat_messages+71)
 extern struct rules *rulepool[];
+
 static void selecttopic(Widget w, XtPointer number, XtPointer garbage) {
-    char filename[200];
+    char filename[256];
+    const char *langdir = get_lang();
     const char *s = XtName(w);
     Arg Args[2];
     int i = atoi(s+4);
-
-    if (TXT_HELPFILES[i][0] != '/')
-	sprintf(filename, "%s/%s/%s", LIBDIR, langdir, TXT_HELPFILES[i]);
-    else
+    
+    if (TXT_HELPFILES[i][0] != '/') {
+        if (*langdir) {
+  	    sprintf(filename, "%s/%s/%s", LIBDIR, langdir, TXT_HELPFILES[i]);
+	    if (access(filename, R_OK))
+	        langdir = "";
+	}
+        if (!*langdir)
+	    sprintf(filename, "%s/%s", LIBDIR, TXT_HELPFILES[i]);
+    } else {
 	strcpy(filename, TXT_HELPFILES[i]);
-
+    }
     XtSetArg(Args[0], XtNstring, filename);
     XtSetArg(Args[1], XtNtype, XawAsciiFile);
     XtSetValues(helptext, Args, 2);
