@@ -17,6 +17,8 @@
 #include "pat.bm"
 #include "gray1.bm"
 
+extern int calc_yoff(int);
+
 void init_fonts(const char *statusfont, const char *buttonfont)
 {   static const char *buttonfonttable[] = {
 	NULL,
@@ -90,11 +92,10 @@ void init_windows(int ac, char **av, const char *geom,
     B_ROUND_H = brady;
 
     /* compute minimum size needed */
-    generic_minwindow((XSize_t *)&xsh.min_width, (XSize_t *)&xsh.min_height);
-    graphic.width = xsh.min_width;
-    graphic.height = xsh.min_height;
-    graphic.min_width = graphic.width;
-    graphic.min_height = graphic.height;
+    generic_minwindow();
+    geo.min_height += calc_yoff(geo.min_width);
+    xsh.min_width = graphic.width = geo.min_width;
+    xsh.min_height = graphic.height = geo.min_height;
 
     xsh.flags = PPosition | PSize | PMinSize;
     x = y = 0;
