@@ -148,6 +148,7 @@ usage(const char *arg)
     fprintf(stderr, "-cards <cardset>        set cardset to external file\n");
 #endif
     fprintf(stderr, "-xpmdir <directory>     for internal cardset: load xpm files\n");
+    fprintf(stderr, "-xpmcls <value>         set allowed closeness for colour allocation (0 to 65535)\n");
     fprintf(stderr, "-sound <0/1>            enables/disables sound\n");
 
     fprintf(stderr, "-tb <backgroundcolor>   set background color of tableau\n");
@@ -198,7 +199,7 @@ main(int argc, char **argv)
     /* these must be static, since their address is taken */
     static int slots, faceup, facedown, decks, p0, p1, p2, p3; /* rules customization */
     static int linewidth, cround, gapx, gapy, autolayout, mem;
-    static int tmps, arrw, arrh, jokers, bx, by, sound = 1;
+    static int tmps, arrw, arrh, jokers, bx, by, sound = 1, xpmcls = DEFAULT_CLOSENESS;
     static const char *ruleset, *cbcolor, *bkcolor = "DarkKhaki";
     static const char *cardset = NULL, *markcolor, *xpmdir;
     static const char *arrowcolor;
@@ -217,7 +218,8 @@ static struct option {
     { "-rules",	    "Rules",	     "Gypsy",     &ruleset,   String },
     { "-autolayout","AutoLayout",    "1",         &autolayout,Number,  0,  1 },
     { "-mem",       "Mem",           "1",         &mem,       Number,  0,  1 },
-    { "-xpmdir",    "XpmDir",        NULL,        &xpmdir,    String },
+    { "-xpmdir",    "Xpmdir",        NULL,        &xpmdir,    String },
+    { "-xpmcls",    "Xpmcls",        XDEFAULT_CLOSENESS,        &xpmcls,    Number, 0, 65535 },
 #ifdef LOADCARDS
     { "-cards",     "Cards",         NULL,        &cardset,   String },
 #endif
@@ -235,8 +237,8 @@ static struct option {
     { "-sound",     "Sound",         "1",         &sound,     Number,  0,  1 },
     { "-bx",        "ButtonGapX",    "4",         &bx,        Number,  0, 99 },
     { "-by",        "ButtonGapY",    "4",         &by,        Number,  0, 99 },
-    { "-aw",        "ArrowWidth",    NULL,        &arrw,      Number, -1, 30 },
-    { "-ah",        "ArrowHeight",   NULL,        &arrh,      Number, -1, 30 },
+    { "-aw",        "ArrowWidth",    "4",         &arrw,      Number, -1, 30 },
+    { "-ah",        "ArrowHeight",   "4",         &arrh,      Number, -1, 30 },
     { "-gapx",      "GapX",          "4",         &gapx,      Number,  0, 99 },
     { "-gapy",      "GapY",          "4",         &gapy,      Number,  0, 99 },
     { "-cb",        "CardbackColor", "SlateGrey", &cbcolor,   String },
@@ -327,11 +329,6 @@ static struct option {
 	(decks != -1 || slots != -1 || faceup != -1 || facedown != -1))
 	fprintf(stderr, "warning: customization parameters are ignored\n");
 
-    {
-	const char *s;
-
-	langdir = (s = getenv("LANG")) ? s : "";
-    }
     read_message_file("messages");
     read_keyboard_file("keys");
 
@@ -372,17 +369,21 @@ static struct option {
 	xv_get(cms, CMS_X_COLORS, pixels);
     }
     
+#if defined(GFX2) && defined(LOADCARDS)
+    if (!graphic.is_color && !cardset)
+	cardset = "french";	/* use B&W cardset, xpm requires too much colours */
+#endif
+
     init_cards(cardset, cround, cround, pixels[C_RED].pixel,
-	       pixels[C_BACK].pixel, mem, xpmdir);
+	       pixels[C_BACK].pixel, mem, xpmdir, xpmcls);
 
     graphic.xgap = (XSize_t) gapx;
     graphic.ygap = (XSize_t) gapy;
     graphic.autolayout = autolayout;
     /* compute minimum size needed */
-    generic_minwindow((XSize_t *) &graphic.min_width,
-		      (XSize_t *) &graphic.min_height);
-    graphic.width = graphic.min_width;
-    graphic.height = graphic.min_height;
+    generic_minwindow();
+    graphic.width = geo.min_width;
+    graphic.height = geo.min_height;
     mainwindow = (Frame) xv_create(0, FRAME,
 				   XV_INSTANCE_NAME, "xpat",
 #if 0
@@ -428,6 +429,7 @@ static struct option {
 	      PANEL_LABEL_STRING, "Game",
 	      PANEL_ITEM_MENU, mnu,
 	      0);
+    /* the context menu is activated with the right mouse button */
     cmenu = (Menu) xv_create(0, MENU,
 			     MENU_GEN_PIN_WINDOW, mainwindow, "XPat",
 			     MENU_TITLE_ITEM, "XPat",
@@ -537,10 +539,10 @@ static struct option {
     /* frame size forwarding fails when borders are on */
     canvas = (Canvas) xv_create(mainwindow, CANVAS,
 				XV_INSTANCE_NAME, "desktop",
-				CANVAS_MIN_PAINT_WIDTH, graphic.min_width,
-				CANVAS_MIN_PAINT_HEIGHT, graphic.min_height,
-				CANVAS_WIDTH, graphic.min_width,
-				CANVAS_HEIGHT, graphic.min_height,
+				CANVAS_MIN_PAINT_WIDTH, geo.min_width,
+				CANVAS_MIN_PAINT_HEIGHT, geo.min_height,
+				CANVAS_WIDTH, geo.min_width,
+				CANVAS_HEIGHT, geo.min_height,
 				CANVAS_RESIZE_PROC, table_resize,
 				CANVAS_REPAINT_PROC, table_repaint,
 				WIN_BELOW, buttonbox,
