@@ -12,6 +12,13 @@
 /*****************************************************************************/
 #include "xpatgame.h"
 
+#include <limits.h>
+#ifndef _POSIX_NAME_MAX
+#  undef _POSIX_SOURCE		/* seems that we have no POSIX system! */
+#else
+#  include "unistd.h"		/* for access() */
+#endif
+
 static const char
     n_ace[] = "Ace", n_deuce[] = "Deuce", n_three[] = "Three",
     n_four[] = "Four", n_five[] = "Five", n_six[] = "Six", n_seven[] = "Seven",
@@ -103,8 +110,8 @@ const char *xpat_messages[] = {
     "",
     "",
     "",
-    "help.Spider",	/* we keep the names to 14 characters. Not nice, but POSIX */
-    "help.Gypsy",
+    "help.Spider",	/* we limit the filenames to 14 characters. */
+    "help.Gypsy",	/* Not nice, but POSIX conforming */
     "help.Klondike",
     "help.Seahaven",
     "help.FreeCell",	/* 80 */
@@ -117,8 +124,8 @@ const char *xpat_messages[] = {
     "help.Canfield",
     "help.Cotillion",
     "hlpBakersDozen",
-    "",
-    "",
+    "help.Oonsoo",
+    "help.BlueMoon",
     "Please wait, loading cards...",
     "Welcome to XPat2 version %s!",
     "",
@@ -167,15 +174,89 @@ char *strsav(const char *txt) {
     return p;
 }
 
+/* map standard language settings to historic xpat ones... */
+const char *get_lang(void) {
+    static char iso_code[3] = { 0 };
+    static char *langdir = NULL;
+    char buff[256];
+    if (langdir)
+	return langdir;		/* directory was checked before */
+
+    /* first call, read environment */
+    langdir = getenv("LANG");
+    if (!langdir)
+	langdir = "";		/* set to empty */
+    if (!*langdir)
+	return langdir;		/* nothing to check */
+
+    /* there is something set. Check values as defined */
+    if (strlen(langdir) + strlen(LIBDIR) + 2 > sizeof(buff)) {
+	fprintf(stderr, "xpat2: path too long for LIBDIR + LANG\n(LIBDIR = %s, LANG = %s)\n",
+		LIBDIR, langdir);
+	exit(EXIT_FAILURE);
+    }
+    /* buff has sufficient size */
+    sprintf(buff, "%s/%s", LIBDIR, langdir);
+
+    if (strlen(langdir) < 2)
+	return langdir;		/* no idea how to change, keep it! */
+
+#ifdef _POSIX_SOURCE
+    /* on posix-systems, we can check if the directory is accessible */
+    if (!access(buff, R_OK))	/* check access to directory */
+	return langdir;		/* directory(?) seems to be readable, keep it */
+#endif
+    /* access denied or some other error.
+       Try to map iso-codes to xpat names */
+
+    iso_code[0] = toupper(langdir[0]);
+    iso_code[1] = toupper(langdir[1]);
+    iso_code[2] = '\0';
+    /* map the existing translations (iso3166) to well-known pathnames */
+    if (!strcmp(iso_code, "DE"))
+	return langdir = "german";
+    if (!strcmp(iso_code, "IT"))
+	return langdir = "italian";
+    if (!strcmp(iso_code, "FR"))
+	return langdir = "french";
+    if (!strcmp(iso_code, ""))
+	return langdir = "german";
+    if (!strcmp(iso_code, "RU") || !strcmp(iso_code, "BY"))
+	return langdir = "russian";
+#ifdef _POSIX_SOURCE
+    /* directory wasn't readable, reset it */
+    return langdir = "";
+#else
+    /* access has not been checked, keep directory */
+    return langdir;
+#endif
+}
+
+FILE *lang_fopen(const char *filename) {
+    char pathname[256];
+    const char *langdir = get_lang();
+    FILE *fp;
+    if (strlen(filename) + strlen(langdir) + strlen(LIBDIR) + 3 <= sizeof(pathname)) {
+      /* buffer has sufficient size */
+      if (*langdir) {
+	  sprintf(pathname, "%s/%s/%s", LIBDIR, langdir, filename);
+	  if ((fp = fopen(pathname, "r")))
+	      return fp;
+      }
+    }
+    if (strlen(filename) + strlen(LIBDIR) + 2 <= sizeof(pathname)) {
+        sprintf(pathname, "%s/%s", LIBDIR, filename);
+	if ((fp = fopen(pathname, "r")))
+	    return fp;
+    }
+    return NULL;
+}
+
 void read_message_file(const char *filename) {
     FILE *fp;
     int i;
     char line[256];
-    if (*filename != '/') {
-	sprintf(line, "%s/%s/%s", LIBDIR, langdir, filename);
-	filename = line;
-    }
-    if (!(fp = fopen(filename, "r")))
+    if (!(fp = lang_fopen(filename)))
 	return;		/* Xt philosophy: ignore error */
     for (i = 0; i < sizeof(xpat_messages) / sizeof(const char *); ++i) {
 	char *p;
@@ -244,7 +325,6 @@ static void replace_binding(struct key_action *p, const char *name) {
 }
 
 static struct key_action *global_bindings = NULL;
-const char *langdir = "";
 
 void add_keybinding(struct key_action **cp, const char *cmd, const char *function) {
     /* a NULL pointer for cmd is a wildcard */
@@ -280,11 +360,7 @@ void read_keyboard_file(const char *filename) {
     struct key_action **cp = &global_bindings;
 
     cmd[1] = '\0';	/* 1-char commands currently */
-    if (*filename != '/') {
-	sprintf(line, "%s/%s/%s", LIBDIR, langdir, filename);
-	filename = line;
-    }
-    if (!(fp = fopen(filename, "r"))) {
+    if (!(fp = lang_fopen(filename))) {
 	fprintf(stderr, "FAILED to open keyboard file \"%s\"\n", filename);
 	return;		/* Xt philosophy: ignore error */
     }
