@@ -11,6 +11,15 @@
 /*****************************************************************************/
 #include "xpatgame.h"
 
+static int count_free_slots(Pileindex avoid) {
+    Pileindex i;
+    int empty_slots = 0;
+    for (i = FIRST_SLOT; i <= LAST_SLOT; ++i)
+	if (i != avoid && EMPTY(i))
+	    ++empty_slots;
+    return empty_slots;
+}
+
 static int Seahaven_valid(Cardindex src, Pileindex dstpile) {
     Pileindex srcpile;
     if (!default_movevalid(src, dstpile))
@@ -22,11 +31,17 @@ static int Seahaven_valid(Cardindex src, Pileindex dstpile) {
        number of tmps free */
     srcpile = getpile(src);
     if (game.piletype[srcpile] == Slot && game.piletype[dstpile] == Slot) {
-	int freetmps, cardsmoved;
+	int freetmps, freeslots = 0, cardsmoved;
+	if ((rules.move_bits & ES_MASK) == ES_ALL) {
+	    /* may use empty slots also */
+	    /* this doubles the number of movable cards
+	       with every free slot => power of two */
+	    freeslots = count_free_slots(dstpile);
+	}
 	cardsmoved = INDEX_OF_LAST_CARD(srcpile) + 1 - src;
 	freetmps = rules.numtmps - (INDEX_OF_FIRST_CARD(
             LAST_SLOT+1+rules.numtmps) - INDEX_OF_FIRST_CARD(LAST_SLOT+1));
-	if (cardsmoved-1 > freetmps)
+	if (cardsmoved > (freetmps + 1) * (1 << freeslots))
 	    return 0;
     }
     return 1;
