@@ -11,6 +11,24 @@
 /*****************************************************************************/
 #include "xpatgame.h"
 
+/* you win by completing all suits, either on the stacks */
+/* or (for more points, see Spider_score) in the slots   */
+static int Spider_won(void) {
+    Pileindex i;
+    Cardindex k;
+
+    /* all slots must be empty or contain complete suits */
+    for (i = FIRST_SLOT; i <= LAST_SLOT; i++) {
+	if (!EMPTY(i)) {
+	    if (CARDS_ON_PILE(i) != 13) return 0;
+	    k = INDEX_OF_LAST_CARD(i);
+	    if (!game.visible[k-12]) return 0;
+	    if (complete_suit(i, k) == -1) return 0;
+	}
+    }
+    return 1;
+}
+
 /* calculate score according to Spider documentation */
 static int Spider_score(void) {
     Pileindex i;
@@ -28,7 +46,7 @@ static int Spider_score(void) {
 	total_visible = 1;
 	for (j = game.ind[i]; j < INDEX_OF_LAST_CARD(i); j++) {
 	    if (game.visible[j] &&
-		in_strong_sequence(game.cards[j], game.cards[j+1])) {
+               in_strong_sequence(game.cards[j+1], game.cards[j])) {
 		/* visible combined cards give 2 Points */
 		score += 2;
 	    }
@@ -109,7 +127,7 @@ struct rules Spider_rules = {
     1,		/* faceup */
     0,		/* newgame_bits */
     NULL,	/* new_game */
-    NULL,	/* game_won */
+    Spider_won,        /* game_won */
     NULL,	/* new_cards */
     ES_ALL|US_R|MG_RS|ST_13, /* move_bits */
     NULL,	/* deal_cards */
