@@ -12,16 +12,17 @@
 #include "xpatgame.h"
 #include <time.h>
 
+/* return Cardindex if complete sequence available, -1 if not */
 Cardindex complete_suit(Pileindex pile, Cardindex i) {
     Rank r;
     Suit s;
     int c;
     if (IS_JOKER(game.cards[i]))
-	return 0;
+       return -1;
     r = RANK(game.cards[i]);
     s = SUIT(game.cards[i]);
     if (r != Ace)
-	return 0;
+       return -1;
     for (c = rules.cards_per_color-1; c > 0; --c) {
 	if (--i < game.ind[pile] || !game.visible[i])
 	    return -1;
@@ -184,7 +185,7 @@ int default_movevalid(Cardindex src, Pileindex dstpile) {
 	} else {
 	    switch (rules.move_bits & ST_MASK) {
 	    case ST_NONE:
-		return 0;	/* user may not may cards to stack */
+		return 0;	/* user may not move cards to stack */
 	    case ST_ONE:
 		if (src != INDEX_OF_LAST_CARD(srcpile) || IS_JOKER(game.cards[src]))
 		    return 0;
@@ -204,12 +205,17 @@ int default_movevalid(Cardindex src, Pileindex dstpile) {
     case Slot:
 	if (dstcard == -1) {
 	    switch (rules.move_bits & ES_MASK) {
-		case ES_KINGS:  /* Klondike: only Kings are */
+	    case ES_KINGS:	/* Klondike: only Kings are allowed */
 		if (IS_JOKER(game.cards[src]) || RANK(game.cards[src]) == King)
 		    return 1;
 		break;
+	    case ES_CLUBS:	/* Oonsoo: only clubs are allowed */
+		if (IS_JOKER(game.cards[src]) || !SUIT(game.cards[src]))
+		    return 1;
+		break;
 	    case ES_ALL:
 		return 1;
+	    /* ES_NONE: falls through, not allowed */
 	    }
 	} else {	/* move to non-empty slot */
 	    return in_relaxed_sequence(srccard, dstcard);
@@ -342,6 +348,7 @@ int std_good_hint(Cardindex srcindex, Pileindex dstpile) {
 	    case ES_NONE:
 		value -= 10;	/* no bonus */
 		break;
+	    case ES_CLUBS:
 	    case ES_KINGS:
 		value += 10;
 		break;
