@@ -29,7 +29,15 @@
 /*									     */
 /*	02-Apr-1994: Reworked and commented this file, with xpat version 2   */
 /*			in mind.					     */
+/*	*-Apr-1999: extended for Qt widget set (implies C++ components)	     */
 /*									     */
+/*	Structure of the sources:					     */
+/*	- [a-z]*.c are pure ANSI-C / Posix.1 files; they contain as much     */
+/*		of the logic as possible.				     */
+/*		Among these, r_*.c contain code specfic to certain rule sets */
+/*		Files which include xpatgeo.h cope with the layout on a	     */
+/*		windowing system, but are not specific to one of them.	     */
+/*	- [A-Z]*.c are interface-specific.				     */
 /*									     */
 /*****************************************************************************/
 #include <stdio.h>
@@ -64,8 +72,7 @@ struct key_action {
     void (*action)(void);
     struct key_action *next;
 };
-
-extern const char *langdir;	/* a slash, then the name of a subdir */
+FILE *lang_fopen(const char *filename);
 
 #define True		1
 #define False		0
@@ -180,6 +187,7 @@ extern struct rules {
 				   (Midnight Oil) */
 #define ES_KINGS	0x0001	/* only kings may go there (Klondike) */
 #define ES_ALL		0x0002	/* all cards are allowed (Gypsy, Spider) */
+#define ES_CLUBS	0x0003	/* only clubs suit is allowed (Oonsoo) */
 
 #define US_MASK		0x000c	/* moves to used slots */
 #define US_NONE		0x0000	/* no move to used slots allowed */
@@ -247,10 +255,13 @@ extern struct rules {
     int inited;			/* set to nz when rules are used */
     void (*initfunc)(void);	/* function to be called if rules are inited */
     struct key_action *local_bindings;
+    /* calculate minimum window size: */
+    void (*special_minwindow)(int, int, int, int, int *, int *);
+    /* define layout for given window size (with knowledge of geo): */
+    int (*special_layout)(void);
 } rules;
 
 
-
 /* only parts of this structure are saved: */
 extern struct game {
     long seed;			/* seed of this game			   */
@@ -290,7 +301,7 @@ extern struct game {
 #define INDEX_OF_LAST_CARD(i)	(game.ind[(i)+1]-1)
 #define	INDEX_OF_FIRST_CARD(i)	game.ind[i]
 #define STACK(i)		(i)
-#define SLOT(i)			((i) + rules.numstacks)
+#define XPATSLOT(i)		((i) + rules.numstacks)
 #define IDECK			(game.numpiles-1)
 #define VDECK			(game.numpiles-2)
 #define IS_STACK(i)		((i) < rules.numstacks)
@@ -308,23 +319,33 @@ extern struct game {
 
 /* Prototypes of the Xlib interface / widget that are known to the game: */
 /* (these should go into a different .h file) */
-#if SOUND
+#ifdef SOUND
 int checksound(void);
 void play_sound(const char *);
 #else
 #define play_sound(x)
 #endif
+void flush_display(void);
 void move_card_data(Cardindex dst, Cardindex src);	/* cards.c */
 void draw_pileupdate(Pileindex pile, int delta);
 boolean pile_resize(Pileindex);
 void new_rules_notify(void);
+void new_rules_coming(void);
 void win_game(void);					/* ? */
-void refresh_screen(void); /* request complete redraw */ /* ? */
-void show_arrow(boolean);  /* show/hide the hint arrow */
+void refresh_screen(void);	/* request complete redraw */ /* ? */
+void show_arrow(int);		/* hide/show/redisplay the hint arrow */
 void show_mark(boolean);	/* markcard.c */
 void show_message(const char *, ...);
 void request_confirm(void (*dofunc)(void), const char *prompt);
 
+
+/* layout.c */
+void indep_generic_minwindow(int *, int *, int *, int *);
+int generate_layout(void);
+Cardindex FindCardOfMousePos(Pileindex pileno, int mx, int my);
+void generic_minwindow(void);
+void obtain_current_geometry(void);	/* U */
+
 /* Prototypes of the game that are known to the Xlib interface / widget: */
 
 typedef enum { Disable, Enable, EnableAndRedraw } GraphicsControl;
@@ -377,6 +398,7 @@ void cmd_RotateUp(void);
 void cmd_RotateDown(void);
 
 /* Messages -- internationalisation */
+const char *get_lang(void);
 void read_message_file(const char *filename);
 void read_keyboard_file(const char *filename);
 void add_keybinding(struct key_action **cp, const char *cmd, const char *function);
@@ -457,7 +479,6 @@ extern const char *xpat_messages[];
 #define TXT_CARDSLEFT		(xpat_messages[68])
 #define TXT_HELP_KEYS		(xpat_messages[69])
 #define TXT_HELP_RULES		(xpat_messages[70])
-#define TXT_HELPFILES		(xpat_messages+71)
 
 
 
@@ -480,3 +501,6 @@ extern const char *xpat_messages[];
 #define TXT_SCOREBLOCK		120
 
 extern int rng;
+
+#define SLOTFACTOR 3	/* minimum std height of a slot, measured in units
+			   (CARD_HEIGHT+graphic.ygap) */
